#include "bsp.h"
#include "tty.h"

typedef struct {
  word samples[2048][4];
  word sample_counter;
  word trigger_counter;
  word trigger_position;
  word status;
} LogicAnalyzer;

/* status bits */
#define LA_RUN		0x0001
#define LA_TRIGGER	0x0002

typedef union {
  byte b[4];
  long l;
  void *p;
} LongPtr;

static inline void *
long2ptr(long l)
{
  LongPtr lp;
  lp.l = l;
  return lp.p;
}
static inline void *
byte2ptr(byte h, byte m, byte l)
{
  LongPtr lp;
  lp.b[0] = l;
  lp.b[1] = m;
  lp.b[2] = h;
  lp.b[3] = 0;
  return lp.p;
}

#define F_IO_0	p9.b0
#define F_IO_1	p9.b1
#define F_IO_2	p9.b2
#define F_IO_3	p9.b3

extern void dllexit();

#define LA_ADDR 0x00100000L
volatile LogicAnalyzer *la = (LogicAnalyzer *)(LA_ADDR);

#define RAM_ADDR 0x00400000L
volatile word *ram = (word *)(RAM_ADDR);

main()
{
  unsigned int i, j, t, u, done;
  long count, fails;
  int h, m, l;
  unsigned int oi, oj, op=-1;
  unsigned char new_pd9;
  long addr = 0;
  volatile word *r1, *r2;

  p10.b = 0x33;

  F_IO_0 = 1; /* This is RESET ACTIVE HIGH */
  F_IO_1 = 1; /* chip select ACTIVE LOW */
  F_IO_2 = 0; /* increment phase */
  F_IO_3 = 0; /* decrement phase */

  prcr.b = 15;

  pm0.b = 0x05;
  pm1.b = 0x00;
  cm0   = 0x00;
  ds = 0x0f;
  mcd = 0x12; /* divide by 1 */
  //mcd = 0x02; /* divide by 2 */
  //mcd = 0x04; /* divide by 4 */

  ewcr0 = 0x01;
  ewcr1 = 0x01;
  ewcr2 = 0x01;
  ewcr3 = 0x01;

  new_pd9 = pd9.b | 0x0f;
  prcr.b = 0x04;
  pd9.b = new_pd9; /* f_io_* */


  wait_ms (1);

#if 0
  while (tty_ready ())
    tty_readc ();
  cprintf("about to start...\n");
  while (!tty_ready ())
    ;
  tty_readc ();
#endif

  F_IO_0 = 0; /* de-assert reset */
  F_IO_1 = 0; /* chip select ACTIVE LOW */

  cprintf("running...\n");

  //  while (1)
    {
      volatile word *r;
      word a, b, c, d, t;
      word d0, d1, d2;
      word n0, n1, n2;

      cprintf("about to start...\n");
      tty_readc();

#if 0
      la->sample_counter = 0x1234;
      la->trigger_counter = 0xffff;
      la->trigger_position = 0x1111;
      la->status = 0xcc33;
      a = la->sample_counter;
      b = la->trigger_counter;
      c = la->trigger_position;
      d = la->status;
      cprintf("1234 ffff 1111 cc33 : %w %w %w %w\n", a, b, c, d);
#endif


      la->status = LA_RUN;
      la->status = 0;

      r = (word *)((byte *)ram + 1);

      /* ram:  [ 2 ] [ 3 ] [ 4 ]    */
      /* r: [ 1 ] [ 2 ] [ 3 ] [ 4 ] */
      /*          cd ab 34 12 78 56 */

      r[2] = 0xabcd;
      r[3] = 0x1234;
      r[4] = 0x5678;
      a = r[3];   /* 1234 */
      b = r[4];   /* 5678 */
      c = ram[3]; /* 34ab */
      d = ram[4]; /* 7812 */

      la->status = LA_TRIGGER;
      la->status = 0;
      t = la->trigger_position;
      cprintf("%w %w %w %w\n", a, b, c, d);

      d2 = ~0;
      tty_rawc(128);
      for (i=t; i != t+1024; i++)
	{
	  word addr = i & 0x3ff;
	  n0 = la->samples[addr][0];
	  n1 = la->samples[addr][1];
	  n2 = la->samples[addr][2];

#if 0
	  if (n0 == d0 && n1 == d1 && n2 == d2)
	    continue;
#endif
	  d0 = n0;
	  d1 = n1;
	  d2 = n2;
	  
	  //cprintf("%w : %N %W %W\n", i, n2, n1, n0);
	  tty_rawc (i-c);
	  tty_rawc ((i-c) >> 8);
	  tty_rawc (d0);
	  tty_rawc (d0 >> 8);
	  tty_rawc (d1);
	  tty_rawc (d1 >> 8);
	  tty_rawc (d2);
	}
      tty_rawc(255);
      tty_rawc(255);
    }

  prcr.b = 15;
  pm0.pm0mode = 0; /* single chip */
  prcr.b = 0;

}
