#include "bsp.h"
#include "tty.h"

static char buf[16];
static unsigned char bufp, bufg;
static volatile int bufc;

/* nRTS is the only input */
#define nRTS p7.b3

#define nCTS p7.b2

#define STOP()  nCTS = 1
#define START() nCTS = 0

void __attribute__((interrupt))
s1rx_handler ()
{
  buf[bufp&15] = u1rb;
  bufp ++;
  bufc ++;
  if (bufc > 12)
    STOP();
}

void
tty_init ()
{
  unsigned char *id = (unsigned char *)(0xffff00UL);
  pd7.b2 = 1;
  pd7.b3 = 0;
  pd6.b7 = 1;

  START();

  /* Round up */
  u1brg = (CLOCK_HZ /*+ BAUD * 8*/) / (BAUD * 16) - 1;

  u1mr.b = 0x05;	/* UART, 8N1 */
  u1c0.b = 0x10;	/* LSB first, CMOS, buffered, f1 */
  u1c1.b = 0x05;	/* enable tx, rx */

  ps0.b7 = 1;		/* make P6.7 TxD1 */

  s1ric = 5;

  wait_ms (1);

  cprintf("\nID: %b:%b:%b:%b:%b:%b:%b\n",
	  id[0xd7], id[0xe3], id[0xeb], id[0xef], id[0xf3], id[0xf7], id[0xfb]);
}

void
tty_flush ()
{
  while (bufc)
    ;
}

void
tty_rawc (char c)
{
  while (!u1c1.ti)
    ;
  u1tb = c;
}

void
tty_putc (char c)
{
  if (c == '\n')
    tty_rawc ('\r');
  tty_rawc(c);
}

void
tty_puts (char *s)
{
  while (*s)
    {
      tty_putc (*s);
      s++;
    }
}

void
tty_write (void *data, int len)
{
  char *d = (char *)data;
  while (len--)
    tty_putc (*d++);
}

write (int fd, void *data, int len)
{
  tty_write (data, len);
  return len;
}

/* returns number of characters available */
int
tty_ready (void)
{
  return bufc;
}

/* returns character */
int
tty_readc (void)
{
  int rv;
#if 1
  while (bufc == 0)
    ;
  rv = buf[bufg&15];
  bufg++;
  asm("fclr I");
  bufc--;
  if (bufc < 4)
    START();
  asm("fset I");
#else
  while (u1c1.ri)
    ;
  rv = u1rb;
#endif
  return rv;
}

#if 0
/* returns number of characters read */
int
tty_read (void *data, int len)
{
}
#endif
